from re import match

from django.db import models
from django.db.models import Q
from django.http import Http404


class CountryManager(models.Manager):
    def find(self, identifier):
        if type(identifier) is int or match(r'^\d+', identifier):
            return self.get(pk=identifier)
        else:
            return self.get(
                Q(iso2__iexact=identifier) |
                Q(iso3__iexact=identifier) |
                Q(name__iexact=identifier)
            )

    def find_or_404(self, identifier):
        try:
            return self.find(identifier)
        except:
            raise Http404


class TerritoryManager(models.Manager):
    def by_country(self, country=None):
        if type(country) is int or match(r'^\d+$', country):
            return self.filter(country__id=country)

        return self.all() if country == None else self.filter(
            Q(country__iso2__iexact=country) |
            Q(country__iso3__iexact=country) |
            Q(country__name__iexact=country)
        )

    def by_country_iso2(self, iso2=None):
        return self.all() if iso2 == None else self.filter(
            country__iso2__iexact=iso2).order_by('abbr')

    def by_country_iso3(self, iso3=None):
        return self.all() if iso3 == None else self.filter(
            country__iso3__iexact=iso3).order_by('abbr')

    def by_country_id(self, country_id=None):
        return self.all() if country_id == None else self.filter(
            country__id=country_id).order_by('abbr')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('iso2', models.CharField(unique=True, max_length=2, verbose_name='ISO 3166-1 Alpha 2 Name')),
                ('iso3', models.CharField(unique=True, max_length=3, verbose_name='ISO 3166-1 Alpha 3 Name')),
                ('name', models.CharField(unique=True, max_length=128, verbose_name='Country Name')),
            ],
            options={
                'verbose_name_plural': 'Countries',
                'verbose_name': 'Country',
                'ordering': ('iso2', 'name'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Territory',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('abbr', models.CharField(max_length=10, verbose_name='Territory Abbreviation')),
                ('name', models.CharField(max_length=128, verbose_name='Territory Name')),
                ('country', models.ForeignKey(related_name='territories', to='locality.Country')),
            ],
            options={
                'verbose_name_plural': 'Territories',
                'verbose_name': 'Territory',
                'ordering': ('abbr', 'name'),
            },
            bases=(models.Model,),
        ),
    ]



from django.core.validators import EMPTY_VALUES
from django.db.models import Q
from django.forms import fields, ValidationError
from django.utils.translation import ugettext_lazy as _

from locality.models import Country


class CountrySelectField(fields.Field):
    """
    A form field that validates its input to iso2 or iso3 abbreviations
    or the name of countries in the database.
    """
    default_error_messages = {
        'invalid': _("Select a Country."),
    }

    def clean(self, value):
        super(CountrySelectField, self).clean(value)

        if value in EMPTY_VALUES:
            return u''

        country = Country.objects.filter(
            Q(name__iexact=value) | Q(iso2__iexact=value) | Q(
                iso3__iexact=value))

        if not country.exists():
            raise ValidationError(self.default_error_messages['invalid'])

        return country[0]


class CountrySelectWidget(fields.Select):
    """
    A select widget that uses all countries in alphabetical order as its choices.
    """

    def __init__(self, attrs=None):
        choices = []
        for country in Country.objects.all().order_by('name'):
            choices.append((country.iso2, country.name,))
        super(CountrySelectWidget, self).__init__(attrs, choices=choices)



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name="django-locality",
    version="0.2.3",
    url='https://github.com/rfkrocktk/django-locality',
    license='AGPL',
    description='Countries and territories, made usable.',
    author='Naftuli Tzvi Kay',
    author_email='rfkrocktk@gmail.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    package_data={
    '': ['locality/fixtures/*.json']
    },
    install_requires=[
        'django',
        'django-localflavor',
        'setuptools'
    ],
    zip_safe=False,
)



from django.core import serializers
from django.http import HttpResponse
from django.utils.encoding import force_unicode
from django.utils.functional import Promise
from django import VERSION

from locality.models import Country, Territory

version = '%d.%d' % VERSION[:2]
if version < 1.5:
    from django.utils import simplejson as json
else:
    import json


def countries(request, to="json"):
    return HttpResponse(serializers.serialize(to, Country.objects.all()),
                        mimetype="application/%s; charset=utf-8" % to)


def territories(request, to="json"):
    return HttpResponse(serializers.serialize(to, Territory.objects.all()),
                        mimetype="application/%s; charset=utf-8" % to)


def territories_by_country(request, country=None, to="json"):
    return HttpResponse(
        serializers.serialize(to, Territory.objects.by_country(country)),
        mimetype="application/%s; charset=utf-8" % to)


def country(request, country=None, to="json"):
    return HttpResponse(
        serializers.serialize(to, [Country.objects.find(country)]),
        mimetype="application/%s; charset=utf-8" % to)


def generate_territories(request):
    from localflavor.ar import ar_provinces
    from localflavor.au import au_states
    from localflavor.at import at_states
    from localflavor.be import be_provinces
    from localflavor.br import br_states
    from localflavor.ca import ca_provinces
    from localflavor.de import de_states
    from localflavor.nl import nl_provinces
    from localflavor.in_ import in_states
    from localflavor.it import it_province
    from localflavor.mx import mx_states
    from localflavor.es import es_provinces
    from localflavor.ch import ch_states
    from localflavor.tr import tr_provinces
    from localflavor.us import us_states

    counter = SuperCounter(start=1)
    c = counter

    output = []
    output.extend(create_territories("AR", ar_provinces.PROVINCE_CHOICES, c))
    output.extend(create_territories("AU", au_states.STATE_CHOICES, c))
    output.extend(create_territories("AT", at_states.STATE_CHOICES, c))
    output.extend(create_territories("BE", be_provinces.PROVINCE_CHOICES, c))
    output.extend(create_territories("BR", br_states.STATE_CHOICES, c))
    output.extend(create_territories("CA", ca_provinces.PROVINCE_CHOICES, c))
    output.extend(create_territories("DE", de_states.STATE_CHOICES, c))
    output.extend(create_territories("NL", nl_provinces.PROVINCE_CHOICES, c))
    output.extend(create_territories("IN", in_states.STATE_CHOICES, c))
    output.extend(create_territories("IT", it_province.PROVINCE_CHOICES, c))
    output.extend(create_territories("MX", mx_states.STATE_CHOICES, c))
    output.extend(create_territories("ES", es_provinces.PROVINCE_CHOICES, c))
    output.extend(create_territories("CH", ch_states.STATE_CHOICES, c))
    output.extend(create_territories("TR", tr_provinces.PROVINCE_CHOICES, c))
    output.extend(create_territories("US", us_states.STATE_CHOICES, c))

    return HttpResponse(LazyEncoder(indent="\t").encode(output),
                        mimetype="application/json; charset=utf-8")


def create_territories(country_abbr, territories, counter):
    result = []
    country = Country.objects.get(iso2=country_abbr)

    for territory in territories:
        result.append({
            'model': "locality.territory",
            'pk': counter.get(),
            'fields': {
                'abbr': territory[0],
                'name': territory[1],
                'country': country.id
            }})

        counter.increment()

    return result


class LazyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_unicode(obj)
        return super(LazyEncoder, self).default(obj)


class SuperCounter():
    __current__ = 0

    def __init__(self, start=0):
        self.__current__ = start

    def get(self):
        return self.__current__

    def increment(self):
        self.__current__ = self.__current__ + 1
        return self.__current__



from django.conf.urls.defaults import patterns, url
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns(
    'locality.views',
    # generate initial data from localflavors for territories
    # url(r'^json/generate/territories/?$', 'generate_territories'),

    # get all countries
    url(r'^(?P<to>(?:json|xml|yaml))/countries/(?:all/?)?$', 'countries'),
    # get all territories
    url(r'^(?P<to>(?:json|xml|yaml))/territories/(?:all/?)?$', 'territories'),
    # get territories by country
    url(r'^(?P<to>(?:json|xml|yaml))/territories(?:/(?P<country>(?:\d+|\w{2}|\w{3}))/?)?$',
        'territories_by_country'),
    # get country
    url(r'^(?P<to>(?:json|xml|yaml))/country/(?P<country>(?:\d+|\w{2}|\w{3}))/?$',
        'country'),
    #url(r'^admin/', include(admin.site.urls)),
)



from django.contrib import admin
from django.db import models
from django.utils.translation import ugettext_lazy as _

from locality import managers


class Country(models.Model):
    iso2 = models.CharField('ISO 3166-1 Alpha 2 Name', max_length=2,
                            unique=True)
    iso3 = models.CharField('ISO 3166-1 Alpha 3 Name', max_length=3,
                            unique=True)
    name = models.CharField('Country Name', max_length=128, unique=True)

    objects = managers.CountryManager()

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    @property
    def abbr(self):
        return self.iso2

    class Meta:
        verbose_name = _('Country')
        verbose_name_plural = _('Countries')
        ordering = ('iso2', 'name',)


class CountryAdmin(admin.ModelAdmin):
    list_display = ('iso2', 'iso3', 'name',)


class Territory(models.Model):
    abbr = models.CharField('Territory Abbreviation', max_length=10)
    name = models.CharField('Territory Name', max_length=128)
    country = models.ForeignKey('Country', related_name="territories",
                                on_delete=models.CASCADE)

    objects = managers.TerritoryManager()

    def __str__(self):
        return "%s, %s" % (self.name, self.country.name)

    def __unicode__(self):
        return u"%s, %s" % (self.name, self.country.name)

    class Meta:
        verbose_name = _('Territory')
        verbose_name_plural = _('Territories')
        ordering = ('abbr', 'name',)


class TerritoryAdmin(admin.ModelAdmin):
    list_display = ('abbr', 'country', 'name',)


admin.site.register(Country, CountryAdmin)
admin.site.register(Territory, TerritoryAdmin)
